/*
 *	SCCS: @(#)dynlink.c	1.1 (98/09/01)
 *
 *	UniSoft Ltd., London, England
 *
 * Copyright (c) 1998 The Open Group
 * All rights reserved.
 *
 * No part of this source code may be reproduced, stored in a retrieval
 * system, or transmitted, in any form or by any means, electronic,
 * mechanical, photocopying, recording or otherwise, except as stated
 * in the end-user licence agreement, without the prior permission of
 * the copyright owners.
 * A copy of the end-user licence agreement is contained in the file
 * Licence which accompanies this distribution.
 * 
 * Motif, OSF/1, UNIX and the "X" device are registered trademarks and
 * IT DialTone and The Open Group are trademarks of The Open Group in
 * the US and other countries.
 *
 * X/Open is a trademark of X/Open Company Limited in the UK and other
 * countries.
 *
 */

#ifndef lint
static char sccsid[] = "@(#)dynlink.c	1.1 (98/09/01) TETware release 3.3";
#endif

/************************************************************************

SCCS:   	@(#)dynlink.c	1.1 98/09/01 TETware release 3.3
NAME:		dynlink.c
PRODUCT:	TETware
AUTHOR:		Andrew Dingwall, UniSoft Ltd.
DATE CREATED:	July 1998

DESCRIPTION:
	this is a simple dynamic linker for use when building a test case
	to use a shared API library on a Win32 system

	the dynamic linker is in two parts:

		tet_w32dynlink() resides in the main program
		tet_w32dlcheck() resides in the shared API library

	tet_w32dynlink() performs the dynamic linking
	tet_w32dlcheck() ensures that none of the pointers have been missed

	see the comment in dtmac.h for an overview of how this works


	no calls to TETware library functions are allowed from this file

MODIFICATIONS:

************************************************************************/

/* TET_SHLIB_SOURCE implies TET_SHLIB */
#if defined(TET_SHLIB_SOURCE) && !defined(TET_SHLIB)
#  define TET_SHLIB
#endif

#if defined(_WIN32) && defined(TET_SHLIB)	/* -START-WIN32-CUT- */

#  ifdef TET_SHLIB_SOURCE
#    define TET_DLCHECK_SOURCE
     /* also makes the TET_W32DLCHECK code visible in dtmac.h */
#  else
#    define TET_DYNLINK_SOURCE
     /* also makes the TET_W32DYNLINK code visible in dtmac.h */
#  endif

#  include <stdio.h>
#  include "dtmac.h"

/*
** the following is a list of all the include files that contain symbols
** declared with TET_EXPORT_FUNC, TET_EXPORT_FUNC_PTR, TET_EXPORT_DATA
** and TET_EXPORT_ARRAY, together with any pre-requsite files
*/

#  include "dtthr.h"
#  include "tet_api.h"
#  include "error.h"
#  include "globals.h"
#  include "tcmfuncs.h"
#  include "apilib.h"

#  ifndef TET_LITE	/* -START-LITE-CUT- */
#    include "dtmsg.h"
#    include "ptab.h"
#    include "server.h"
#    include "server_bs.h"
     /*
     ** The following is transport-specific !!!
     ** But it's OK because we only support the INET transport on Win32 systems
     */
#    include "server_in.h"
#    include "synreq.h"
#  endif			/* -END-LITE-CUT- */


#  ifdef TET_DYNLINK_SOURCE

/*
**	tet_w32dynlink() - fix up all the pointers in the shared API
**		library to point to the functions and data items
**		that have been "exported" from the program code
*/

void tet_w32dynlink()
{
	/*
	** do the dynamic linking
	**
	** this file is automatically generated when the shared API library
	** is built
	*/
#	include "dynlink.gen"

	/* check that we didn't miss any of the pointers */
	tet_w32dlcheck();
}

#  endif /* TET_DYNLINK_SOURCE */

#  ifdef TET_DLCHECK_SOURCE

static int dlerrors;		/* the error counter */

/* static function declarations */
static void report_nullptr PROTOLIST((char *));

/*
**	tet_w32dlcheck() - check that all the pointers in the shared API
**		library were fixed up
**
**	there is no return on error
*/

TET_IMPORT void tet_w32dlcheck()
{
	/*
	** check all the pointers
	**
	** this file is automatically generated when the shared API library
	** is built
	*/
#       include "dlcheck.gen"

	if (dlerrors)
		exit(1);
}

/*
**	report_nullptr() - print a diagnostic if a non-fixed-up pointer
**		is found
*/

static void report_nullptr(name)
char *name;
{
	(void) fprintf(stderr,
		"TCM: tet_w32dynlink() did not fix up pointer to %s\n", name);
	dlerrors++;
}


#  endif /* TET_DLCHECK_SOURCE */

#else						/* -END-WIN32-CUT- */

int tet_dynlink_c_not_used;

#endif						/* -WIN32-CUT-LINE- */

